package anonvpn.anon_next.android.service.gui;

import android.content.Context;
import android.content.Intent;
import android.support.test.InstrumentationRegistry;
import android.support.test.runner.AndroidJUnit4;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.w3c.dom.Document;

import java.io.StringReader;

import anon.crypto.SignatureVerifier;
import anon.infoservice.MixCascade;
import anonvpn.anon_next.android.BuildConfig;
import anonvpn.anon_next.android.ui.main.MainActivity;

import static org.junit.Assert.*;

/**
 * Instrumentation test, which will execute on an Android device.
 *
 * @see <a href="http://d.android.com/tools/testing">Testing documentation</a>
 */
@RunWith(AndroidJUnit4.class)
public class AndroidGUIInterfaceTest {

    @Test
    public void chosenCascadeStorageAndRecovery() throws Exception {
        // Context of the app under test.
        Context appContext = InstrumentationRegistry.getTargetContext();

        // Prepare application
        Intent intent = new Intent(appContext, MainActivity.class);
        intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
        appContext.startActivity(intent);


        SignatureVerifier.getInstance().setCheckSignatures(false);

        try {
            String xml = "        <MixCascade context=\"jondonym\" id=\"F30905FCD73B6B30CB5FEFD3250FD66EF4B32591\" serial=\"1500154699723\">\n" +
                    "            <Name>Dresden-Dresden</Name>\n" +
                    "            <Network>\n" +
                    "                <ListenerInterfaces>\n" +
                    "                    <ListenerInterface>\n" +
                    "                        <Type>RAW/TCP</Type>\n" +
                    "                        <Port>6544</Port>\n" +
                    "                        <Host>141.76.46.165</Host>\n" +
                    "                        <IP>141.76.46.165</IP>\n" +
                    "                    </ListenerInterface>\n" +
                    "                </ListenerInterfaces>\n" +
                    "            </Network>\n" +
                    "            <Mixes count=\"2\">\n" +
                    "                <Mix id=\"F30905FCD73B6B30CB5FEFD3250FD66EF4B32591\" serial=\"1495732588740\">\n" +
                    "                    <MixType>FirstMix</MixType>\n" +
                    "                    <Name>First Mix of Dresden-Dresden</Name>\n" +
                    "                    <Location>\n" +
                    "                        <City>Dresden</City>\n" +
                    "                        <State>Saxony</State>\n" +
                    "                        <Country>de</Country>\n" +
                    "                        <Position>\n" +
                    "                            <Geo>\n" +
                    "                                <Longitude>13.762</Longitude>\n" +
                    "                                <Latitude>51.053</Latitude>\n" +
                    "                            </Geo>\n" +
                    "                        </Position>\n" +
                    "                    </Location>\n" +
                    "                    <Operator>\n" +
                    "                        <Organisation>JAP-Team</Organisation>\n" +
                    "                        <URL>http://anon.inf.tu-dresden.de/</URL>\n" +
                    "                        <Country>de</Country>\n" +
                    "                        <OrganisationalUnit>Department of Computer Science, Institute for System Architecture</OrganisationalUnit>\n" +
                    "                    </Operator>\n" +
                    "                    <ListenerInterfaces>\n" +
                    "                        <ListenerInterface>\n" +
                    "                            <NetworkProtocol>RAW/TCP</NetworkProtocol>\n" +
                    "                            <Host>141.76.46.165</Host>\n" +
                    "                            <IP>141.76.46.165</IP>\n" +
                    "                            <Port>6544</Port>\n" +
                    "                        </ListenerInterface>\n" +
                    "                    </ListenerInterfaces>\n" +
                    "                    <Software>\n" +
                    "                        <Version>00.11.17</Version>\n" +
                    "                    </Software>\n" +
                    "                    <PaymentReminderProbability>-1</PaymentReminderProbability>\n" +
                    "                    <LastUpdate>1500155540078</LastUpdate>\n" +
                    "                </Mix>\n" +
                    "                <Mix id=\"75ACF4F101510607BA3E9E348821D8697BE8FC58\" serial=\"1495732671831\">\n" +
                    "                    <MixType payment=\"False\">LastMix</MixType>\n" +
                    "                    <Name>Last Mix of Dresden-Dresden</Name>\n" +
                    "                    <Location>\n" +
                    "                        <City>Dresden</City>\n" +
                    "                        <State>Saxony</State>\n" +
                    "                        <Country>de</Country>\n" +
                    "                        <Position>\n" +
                    "                            <Geo>\n" +
                    "                                <Longitude>13.762</Longitude>\n" +
                    "                                <Latitude>51.053</Latitude>\n" +
                    "                            </Geo>\n" +
                    "                        </Position>\n" +
                    "                    </Location>\n" +
                    "                    <Operator>\n" +
                    "                        <Organisation>JAP-Team</Organisation>\n" +
                    "                        <URL>http://anon.inf.tu-dresden.de/</URL>\n" +
                    "                        <Country>de</Country>\n" +
                    "                        <OrganisationalUnit>Department of Computer Science, Institute for System Architecture</OrganisationalUnit>\n" +
                    "                    </Operator>\n" +
                    "                    <ListenerInterfaces>\n" +
                    "                        <ListenerInterface>\n" +
                    "                            <NetworkProtocol>RAW/TCP</NetworkProtocol>\n" +
                    "                            <Port>46544</Port>\n" +
                    "                            <Host>141.76.45.33</Host>\n" +
                    "                        </ListenerInterface>\n" +
                    "                    </ListenerInterfaces>\n" +
                    "                    <Proxies>\n" +
                    "                        <Proxy>\n" +
                    "                            <VisibleAddresses>\n" +
                    "                                <VisibleAddress>\n" +
                    "                                    <Host>141.76.45.33</Host>\n" +
                    "                                </VisibleAddress>\n" +
                    "                                <VisibleAddress>\n" +
                    "                                    <Host>141.76.45.34</Host>\n" +
                    "                                </VisibleAddress>\n" +
                    "                                <VisibleAddress>\n" +
                    "                                    <Host>141.76.45.35</Host>\n" +
                    "                                </VisibleAddress>\n" +
                    "                                <VisibleAddress>\n" +
                    "                                    <Host>141.76.45.33</Host>\n" +
                    "                                </VisibleAddress>\n" +
                    "                                <VisibleAddress>\n" +
                    "                                    <Host>141.76.45.34</Host>\n" +
                    "                                </VisibleAddress>\n" +
                    "                                <VisibleAddress>\n" +
                    "                                    <Host>141.76.45.35</Host>\n" +
                    "                                </VisibleAddress>\n" +
                    "                            </VisibleAddresses>\n" +
                    "                        </Proxy>\n" +
                    "                    </Proxies>\n" +
                    "                    <Software>\n" +
                    "                        <Version>00.11.19</Version>\n" +
                    "                    </Software>\n" +
                    "                    <LastUpdate>1500155542631</LastUpdate>\n" +
                    "                    <MixProtocolVersion>0.6</MixProtocolVersion>\n" +
                    "                    <FlowControl withUpstreamFlowControl=\"true\">\n" +
                    "                        <UpstreamSendMe>80</UpstreamSendMe>\n" +
                    "                        <DownstreamSendMe>80</DownstreamSendMe>\n" +
                    "                    </FlowControl>\n" +
                    "                    <RSAKeyValue>\n" +
                    "                        <Modulus>rB4/VxKrBWSHLUIYY/Pq2qRuq8k+NrnM4sex4R42BzU2rlVnB8EPNnAMgfCxpUFw\n" +
                    "62Rvvxr22uUTb1xcDWaBEcjC4W9if3KVMwru40kfG+p77+G0/mSURwQYGGT9JzBe\n" +
                    "3pb/Yo9eLnerTtCXe7pNZgC6zXPThnz3ODo494/FaFU=</Modulus>\n" +
                    "                        <Exponent>AQAB</Exponent>\n" +
                    "                    </RSAKeyValue>\n" +
                    "                    <Compatibility version=\"00.11.19\">\n" +
                    "                        <Flags>\n" +
                    "                            <NewFlowControl></NewFlowControl>\n" +
                    "                            <NewChannelEncryption></NewChannelEncryption>\n" +
                    "                            <WithIntegrityCheck></WithIntegrityCheck>\n" +
                    "                        </Flags>\n" +
                    "                    </Compatibility>\n" +
                    "                    <Nonce>1d+zYXdF5RlaU2l30wBKPQ==</Nonce>\n" +
                    "                    <KeepAlive>\n" +
                    "                        <SendInterval>65000</SendInterval>\n" +
                    "                        <ReceiveInterval>75000</ReceiveInterval>\n" +
                    "                    </KeepAlive>\n" +
                    "                </Mix>\n" +
                    "            </Mixes>\n" +
                    "            <Payment required=\"false\"></Payment>\n" +
                    "            <LastUpdate>1500993297300</LastUpdate>\n" +
                    "        </MixCascade>";
            Document doc = anon.util.XMLUtil.readXMLDocument(new StringReader(xml));
            MixCascade testCascade = new MixCascade(doc.getDocumentElement());
        } catch(Exception e){
            e.printStackTrace();
        }

    }
}
