package anonvpn.anon_next.android;

/**
 * Created by migue on 16.06.15.
 * handles a vpn connection
 */

import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.net.VpnService;
import android.os.IBinder;
import android.os.ParcelFileDescriptor;
import android.util.Log;

import java.io.IOException;

import anonvpn.anon_next.android.service.gui.AndroidGUIFactory;
import anonvpn.anon_next.android.service.gui.AndroidGUIInterface;
import anonvpn.anon_next.android.service.networking.AndroidNetworkFactory;
import anonvpn.anon_next.android.service.networking.AndroidProtectedSocketFactory;
import anonvpn.anon_next.android.service.persistence.AndroidPersistenceFactory;
import anonvpn.anon_next.android.util.AndroidResourceLoaderHelper;
import anonvpn.anon_next.core.ANONVPNService;

public class AndroidVpnService extends VpnService
	{

		static
			{
				System.loadLibrary("FileDescriptorHandler");
			}

		ANONVPNService anonvpnService = null;

		// This reference is needed to properly shutdown the service later on
		ParcelFileDescriptor mTunnelingDevice = null;

		// TODO: Find a way to adapt the MTU to the block size of the ANON protocol
		int TUNNELING_MTU = 1500;

		AndroidNetworkFactory mNetworkFactory = null;

		private native int setBlocking(int fd_id);

		/**
		 * We expect this method to be called after VPNService.prepare has been called.
		 *
		 * @param intent
		 * @param flags
		 * @param startId
		 * @return
		 */
		@Override
		public int onStartCommand(Intent intent, int flags, int startId)
			{

				// onStartCommand may be run multiple times (e.g. if the user turns the smartphone)
				if (anonvpnService == null)
					{
						Log.e("AndroidVPNService", Thread.currentThread().getId() + "Setting up tunneling device!");

						mNetworkFactory = new AndroidNetworkFactory(this, TUNNELING_MTU);
						AndroidPersistenceFactory persistenceFactory = null;
						AndroidProtectedSocketFactory mixSocketFactory = new AndroidProtectedSocketFactory(this);
						try {
							persistenceFactory = new AndroidPersistenceFactory(getSharedPreferences("ANONdroidVPN.settings", Context.MODE_PRIVATE));
						} catch (Exception e) {
							e.printStackTrace();
						}
						AndroidGUIFactory GUIFactory = new AndroidGUIFactory(this);

						Log.e("AndroidVPNService", "Starting core service!");
						AndroidResourceLoaderHelper resourecLoaderHelper=new AndroidResourceLoaderHelper(getAssets());
						AndroidLog logger=new AndroidLog();
						anonvpnService = new ANONVPNService(mNetworkFactory, persistenceFactory, GUIFactory, mixSocketFactory,resourecLoaderHelper,logger);
					}

				// service running until it is explicitly stopped,return sticky.
				return START_STICKY;
			}

		public void startTunnelingDevice() {

			byte[]ip=new byte[4];
			ip[0]=10;
			int iIP=10;
			for(int i=1;i<4;i++)
				{
					ip[i]=(byte)(2 + (int) (Math.random() * 252));
					iIP<<=8;
					iIP|=(ip[i]&0x00FF);
				}
			mTunnelingDevice = setupTunnelingDevice(TUNNELING_MTU,ip);
			mNetworkFactory.setTunnelingDescriptor(mTunnelingDevice,iIP);

		}

		public void stopTunnelingDevice() {

			try
			{
				mTunnelingDevice.close();
				mTunnelingDevice = null;
				mNetworkFactory.setTunnelingDescriptor(null,-1);
			}
			catch (Exception e)
			{
				Log.e("AndroidVPNService", "Failed to close tunneling device!");
			}


		}

		public boolean tunnelingEnabled() {
			return mTunnelingDevice != null;
		}

		private ParcelFileDescriptor setupTunnelingDevice(int MTU,byte[]ip)
			{
				ParcelFileDescriptor result = null;
				SharedPreferences prefs = getSharedPreferences("ANONdroidVPN.settings", Context.MODE_PRIVATE);
				String strIP = Integer.toString(ip[0]&0x00FF)+".";
				String DNSServer = "10.0.0.1";

				// Generate a random internal IP address
				strIP = strIP + Integer.toString(ip[1]&0x00FF)+".";
				strIP = strIP + Integer.toString(ip[2]&0x00FF)+".";
				strIP = strIP + Integer.toString(ip[3]&0x00FF);

				VpnService.Builder builder = new VpnService.Builder();
				builder.setMtu(MTU);
				builder.addAddress(strIP, 32);
				builder.addRoute("0.0.0.0", 0);
				builder.addDnsServer(DNSServer);
				builder.setSession("ANON Tunneling Interface");

				// TODO: Consider calling builder.setConfigureIntent to make the VPN connection configurable
				//       from the Android settings

				try
					{
						result = builder.establish();
						if (setBlocking(result.getFd()) == -1)
							{
								throw new IOException("Failed to set blocking mode!");
							}
					}
				catch (Exception e)
					{
						Log.i("ANDROIDTUN", "Failed to create tunneling device: " + e.getMessage());
						return null;
					}
				return result;
			}

		@Override
		public void onDestroy()
			{
				super.onDestroy();
				Log.e("AndroidVPNService", "Destroying core service!");
			}

		@Override
		public IBinder onBind(Intent intent)
			{
				return (AndroidGUIInterface) anonvpnService.getGUIInterface();
			}

		public void shutdown()
			{

				stopTunnelingDevice();
				Log.e("AndroidVPNService", "Stopping!");

				stopSelf();

			}

		@Override
		public void onRevoke()
			{
				Log.e("AndroidVPNService", "Got revoked!");
				shutdown();
			}
	};

/*
		public void startAnonVPN(final boolean bUseMixes)
			{
				if (!mStarted)
					{
						final AnonVpnService anonVPNService = this;

						mThread = new Thread(new Runnable()
							{
								@Override
								public void run()
									{
										VPNServiceUpstreamTunnelCreator upstreamTunnel = null;
										try
											{
												Socket socketToProtect = null;

												final InputStream tunnelIn;

												final int mtu;
												if (bUseMixes)
													{
														AnonClient anon = new AnonClient(null); //create AN.ON client

														//--> set to your cascde here!!
														MixCascade mixCascade = new MixCascade(null, null, mServerAddress, mServerPort); //manually select mix cascade
														mixCascade.setAccessControlPreSharedSecret(mCascadeAccessControlCredential);

														SignatureVerifier.getInstance().setCheckSignatures(false); //insecure, of course - but necessary for this simple manually added mix-cascade

														//initialize the client --> establish the connection
														anon.initialize(mixCascade, new SimpleMixCascadeContainer(mixCascade), null, true);


														//create a communication channel (like a connected socket)
														AnonChannel c = anon.createChannel(AnonChannel.VPN);
														m_iOutputBlockSize = c.getOutputBlockSize();
														mtu = m_iOutputBlockSize * 50;
														tunnelIn = c.getInputStream();
														m_tunnelOut = c.getOutputStream();
														IStreamConnection con = anon.getConnectionToMixCascade().getUnderlyingIStreamConnection();
														SocketConnection scon = (SocketConnection) con;
														socketToProtect = scon.getUnderlyingSocket();
													}
												else //direct connection to VPN Server
													{
														InetSocketAddress isa = new InetSocketAddress(mServerAddress, mServerPort);
														SocketChannel sc = SocketChannel.open();
														sc.connect(isa);
														sc.configureBlocking(true);
														m_tunnelOut = sc.socket().getOutputStream();
														tunnelIn = sc.socket().getInputStream();
														socketToProtect = sc.socket();
														m_iOutputBlockSize = 0x00FFFF;
														mtu = 1400;
													}

												if (!protect(socketToProtect))
													{
														throw new IllegalStateException("Cannot protect the tunnel");
													}
												//set a random IP address and start off
												String response = handshake(mtu);
												mStarted = true;
												Message msg = Message.obtain(m_IncomingMessagesHandler,MESSAGE_IP);
												//Transparancy, telling the user which IP is broadcasting when browsing
												msg.obj = "VPN Connected, your new IP: " + response;
												msg.sendToTarget();

												upstreamTunnel = new VPNServiceUpstreamTunnelCreator(mInterface, anonVPNService);

												FileOutputStream out = new FileOutputStream(mInterface.getFileDescriptor());
												//ByteBuffer packetIn=ByteBuffer.allocate(32767);
												byte[] arPacketIn = new byte[0x00FFFF];

												//packetIn.position(0);
												//packetIn.limit(4);
												int lengthIn = 0, tcpudpdataHeaderlength = 0;
												int posIn = 0;
												int remaining = 4;
												while (m_bRun)
													{
														// Read the incoming packet from the tunnel.
														if (remaining > 0)
															{
																int l = tunnelIn.read(arPacketIn, posIn, remaining);
																remaining -= l;
																posIn += l;
															}
														if (posIn == 4 && lengthIn == 0)
															{
																//AnonIpHeaderWrapper ipheader = new AnonIpHeaderWrapper(arPacketIn);
																lengthIn = ((arPacketIn[2] & 0xFF) << 8) | (arPacketIn[3] & 0xFF);
																remaining = lengthIn - 4;
																int l = tunnelIn.read(arPacketIn, posIn, remaining);
																remaining -= l;
																posIn += l;
															}
														if (lengthIn > 0)
															{
																if (remaining > 0)
																	{
																		int l = tunnelIn.read(arPacketIn, posIn, remaining);
																		remaining -= l;
																		posIn += l;
																	}
																else
																	{
																		out.write(arPacketIn, 0, lengthIn);
																		posIn = 0;
																		remaining = 4;
																		lengthIn = 0;
																	}
															}

													}

											}
										catch (Exception e)
											{
												// Catch any exception
												e.printStackTrace();
											}
										finally
											{
												try
													{
														if (mInterface != null)
															{
																mInterface.close();
																mInterface = null;
																mbroadcastReceiver.unregister();
															}
														if (upstreamTunnel != null)
															{
																upstreamTunnel.stop();
															}
														//sc.close();
														mStarted = false;
														if(m_OutgoingMessenger!=null)
															{
																Message msg = Message.obtain(null,MESSAGE_ANON_SERVICE_STOPPED);
																m_OutgoingMessenger.send(msg);
															}
													}
												catch (Exception e)
													{
														e.printStackTrace();
													}
											}
									}

							}, "ANONdroid service - Receiving");
						//start the service
						Log.i("CHECK", "VPN started");
						m_bRun = true;
						mThread.start();

					}
			}
			*/

/*
		@Override
		public boolean handleMessage(Message message)
			{
				if (message != null)
					{
						if(message.what==AnonVpnService.MESSAGE_IP)
							{
								Toast.makeText(this, (String) message.obj, Toast.LENGTH_LONG).show();
							}
						else if(message.what==AnonVpnService.MESSAGE_REGISTER)
							{
								m_OutgoingMessenger=message.replyTo;
							}
					}
				return true;
			}
*/
/*
		public void sendPacket(byte[] arPacket, int length)
			{
				int index = 0;
				while (length > 0)
					{
						int i = Math.min(this.m_iOutputBlockSize, length);
						try
							{
								m_tunnelOut.write(arPacket, index, i);
							}
						catch (IOException e)
							{
								e.printStackTrace();
								return;
							}
						index += i;
						length -= i;
					}

			}

		;
*/
