package anonvpn.anon_next.android.service.foreground;

import android.app.Notification;
import android.app.PendingIntent;
import android.app.Service;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.os.Handler;
import android.os.IBinder;
import android.support.v4.app.NotificationCompat;
import android.util.Log;

import anonvpn.anon_next.android.AndroidVpnService;
import anonvpn.anon_next.android.R;
import anonvpn.anon_next.android.service.gui.AndroidGUIInterface;
import anonvpn.anon_next.android.ui.main.MainActivity;


public class ForegroundService extends Service {
    private static final String LOG_TAG = "ForegroundService";

    String cascade_ip = "";
    String cascade_location = "";
    String cascade_name = "";
    int Tx = -1;
    int Rx = -1;
    int conn_state = 0;

    private final int TRAFFIC_STATS_DELAY = 3000;
    private final Handler mTrafficStatisticsHandler = new Handler();
    private final GetTrafficStatisticsTask mTrafficStatisticsTask = new GetTrafficStatisticsTask();

    private ServiceConnection mVPNConnection = null;
    private AndroidGUIInterface mServiceInterface = null;

    private class GetTrafficStatisticsTask implements Runnable {
        public boolean isScheduled = false;

        public void run() {

            // This tasks starts right after the VPN service gets started.
            // Therefore, the service might still be starting (hence, the interface is not ready)
            // when this run-procedure gets called.
            if (mServiceInterface != null) {
                Tx = mServiceInterface.getTx();
                Rx = mServiceInterface.getRx();
                Log.w(LOG_TAG, "Got: " + Tx + ", " + Rx + "");

                showNotification();

                mTrafficStatisticsHandler.postDelayed(this, TRAFFIC_STATS_DELAY);
            }
        }
    }

    protected class VPNServiceConnection implements ServiceConnection
    {

        @Override
        public void onServiceConnected(ComponentName name, IBinder service)
        {
            Log.e(LOG_TAG, "Connected to service!");
            mServiceInterface = (AndroidGUIInterface) service;
            mTrafficStatisticsHandler.post(mTrafficStatisticsTask);
            mTrafficStatisticsTask.isScheduled = true;

        }

        @Override
        public void onServiceDisconnected(ComponentName name)
        {
            Log.e(LOG_TAG, "Disconnected from service!");
            mTrafficStatisticsHandler.removeCallbacks(mTrafficStatisticsTask);
            mTrafficStatisticsTask.isScheduled = false;
            mServiceInterface = null;

        }
    }

    @Override
    public void onCreate() {
        super.onCreate();
        Log.i(LOG_TAG, "Received Create Foreground Intent.");
    }

    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
//        if (intent.getAction().equals(Constants.ACTION.STARTFOREGROUND_ACTION)) {
        Log.i(LOG_TAG, "Received Start Foreground Intent.");

        if (intent != null) {
            cascade_ip = intent.getStringExtra("ip");
            cascade_location = intent.getStringExtra("location");
            cascade_name = intent.getStringExtra("name");
            conn_state = intent.getIntExtra("conn_state", 0);
        }
        showNotification();
        return START_STICKY;
    }

    private int getIconError() {
        boolean useWhiteIcon = (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.LOLLIPOP);
        return useWhiteIcon ? R.drawable.anon_jap_noconnection_silhouette : R.drawable.anon_jap_noconnection;
    }

    private int getIconWarning() {
        boolean useWhiteIcon = (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.LOLLIPOP);
        return useWhiteIcon ? R.drawable.anon_jap_warning_silhouette : R.drawable.anon_jap_warning;
    }

    private int getIconOK() {
        boolean useWhiteIcon = (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.LOLLIPOP);
        return useWhiteIcon ? R.drawable.anon_jap_silhouette : R.drawable.anon_jap;
    }

    private void showNotification() {

        NotificationCompat.Builder mBuilder =
                new NotificationCompat.Builder(this)
                        .setContentTitle(getString(R.string.foregroundservice_title));
        if (conn_state == 0)            // Not connected to VPN
        {
            mBuilder.setSmallIcon(getIconError())
                    .setContentText(getString(R.string.foregroundservice_desc_vpn_disabled));

            if (mVPNConnection != null) {
                disconnectVPNService();
            }
        } else if (conn_state == 1)     // Not connected to cascade
        {
            mBuilder.setSmallIcon(getIconWarning())
                    .setContentText(getString(R.string.foregroundservice_desc_cascade_error));

        } else if (conn_state == 2)     // Connection successfully established
        {
            String message = getString(R.string.foregroundservice_desc_active_name) + " " + cascade_name + "\n" +
                    getString(R.string.foregroundservice_desc_active_location) + " " + cascade_location + "\n" +
                    getString(R.string.foregroundservice_desc_active_ip) + " " + cascade_ip + "\n" +
                    getString(R.string.foregroundservice_desc_active_Tx) + " " + Tx + "\n" +
                    getString(R.string.foregroundservice_desc_active_Rx) + " " + Rx;
            mBuilder.setSmallIcon(getIconOK())
                    .setStyle(new NotificationCompat.BigTextStyle().bigText(message))
                    .setContentText(message);

            if (mVPNConnection == null) {
                connectToVPNService();
            }
        } else {
            throw new RuntimeException("Unexpected conn_state: " + conn_state);
        }
        // Creates an explicit intent for an Activity in your app
        Intent resultIntent = new Intent(this, MainActivity.class);

        PendingIntent resultPendingIntent =
                PendingIntent.getActivity(
                        this,
                        0,
                        resultIntent,
                        PendingIntent.FLAG_UPDATE_CURRENT
                );

        mBuilder.setContentIntent(resultPendingIntent);

        Notification notification = mBuilder.build();
        startForeground(1, notification);

    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        Log.i(LOG_TAG, "In onDestroy");

        if (mServiceInterface != null) {
            disconnectVPNService();
        }
//        Toast.makeText(this, "Service Destroyed!", Toast.LENGTH_SHORT).show();
    }

    @Override
    public IBinder onBind(Intent intent) {
        // Used only in case if services are bound (Bound Services).
        return null;
    }

    private void connectToVPNService() {

        if(mVPNConnection !=null)
        {
            throw new AssertionError("Redundant call to connectToVPNService()!");
        }

        mVPNConnection = new VPNServiceConnection();
        Intent serviceIntent = new Intent(getApplicationContext(), AndroidVpnService.class);
        bindService(serviceIntent, mVPNConnection, Context.BIND_ADJUST_WITH_ACTIVITY);
    }

    private void disconnectVPNService() {
        unbindService(mVPNConnection);
        mServiceInterface = null;
        mVPNConnection = null;
    }
}
