package anonvpn.anon_next.android.service.gui;

import android.content.Intent;
import android.os.Binder;
import android.util.Log;

import java.util.Hashtable;

import anon.infoservice.MixCascade;
import anon.util.CountryMapper;
import anonvpn.anon_next.android.AndroidVpnService;
import anonvpn.anon_next.android.service.foreground.ForegroundService;
import anonvpn.anon_next.android.ui.main.MainActivity;
import anonvpn.anon_next.core.InfoService;
import anonvpn.anon_next.core.gui.IVPNInterface;
import anonvpn.anon_next.core.notification.Event;
import anonvpn.anon_next.core.notification.NotificationDispatcher;
import anonvpn.anon_next.core.notification.Observer;
import anonvpn.anon_next.core.persistence.IConfig;


public class AndroidGUIInterface extends Binder implements IVPNInterface, Observer
	{

		private final AndroidVpnService mVpnService;
		private final NotificationDispatcher mDispatcher;
		private final IConfig mConfig;
		private final InfoService mInfoService;
		private static final String LOGTAG = "AndroidGUIInterface";

		private int lastSentBytes = 0;
		private long lastSentUpdate = System.currentTimeMillis();

		private int lastReceivedBytes = 0;
		private long lastReceivedUpdate = System.currentTimeMillis();


		private MixCascade mChosenCascade = null;
		private boolean mConnected = false;

		public AndroidGUIInterface(AndroidVpnService vpnService, NotificationDispatcher dispatcher, IConfig config, InfoService infoService)
			{
				this.mVpnService = vpnService;
				this.mDispatcher = dispatcher;
				this.mConfig = config;
				this.mInfoService = infoService;

				mDispatcher.subscribe(Event.CASCADE_CONNECTED, this);
				mDispatcher.subscribe(Event.CASCADE_DISCONNECTED, this);
				mDispatcher.subscribe(Event.VPN_DISABLED, this);
				mDispatcher.subscribe(Event.VPN_ENABLED, this);

				updateFGService();
			}

		@Override
		public int getTx()
			{
				int sentBytes = mConfig.getSentBytes();
				long currentTime = System.currentTimeMillis();
				int result = 0;

				if (currentTime > lastSentUpdate + 1000L)
					{
						// Handle counter overflow if necessary
						if (sentBytes < lastSentBytes)
							{
								result = ((Integer.MAX_VALUE - lastSentBytes) + sentBytes + 1) / ((int) ((currentTime - lastSentUpdate) / 1000L));
							}
						else
							{
								result = (sentBytes - lastSentBytes) / ((int) ((currentTime - lastSentUpdate) / 1000L));
							}
					}

				lastSentBytes = sentBytes;
				lastSentUpdate = currentTime;
				return result;
			}

		@Override
		public int getRx()
			{
				int receivedBytes = mConfig.getReceivedBytes();
				long currentTime = System.currentTimeMillis();
				int result = 0;

				if (currentTime > lastReceivedUpdate + 1000L)
					{
						// Handle counter overflow if necessary
						if (receivedBytes < lastReceivedBytes)
							{
								result = ((Integer.MAX_VALUE - lastReceivedBytes) + receivedBytes + 1) / ((int) ((currentTime - lastReceivedUpdate) / 1000L));
							}
						else
							{
								result = (receivedBytes - lastReceivedBytes) / ((int) ((currentTime - lastReceivedUpdate) / 1000L));
							}
					}
				lastReceivedBytes = receivedBytes;
				lastReceivedUpdate = currentTime;
				return result;
			}

		@Override
		public boolean isConnectedToCascade()
			{
				// TODO: Replace with actual communication with service
				return mConnected;
			}

		public boolean hasChosenCascade()
			{
				return mConfig.getChosenMixCascade() != null;
			}

		@Override
		public void setChosenCascade(MixCascade cascade, String credential)
			{
				// Save entire cascade to mConfig to retrieve it later
				mConfig.setChosenCascadeCredential(credential);
				mConfig.setChosenMixCascade(cascade);

				Log.i("AndroidVPNService.GUI", Thread.currentThread().getId() + " - Set chosen cascade to cascade with id " + cascade.getId());

				mDispatcher.notify(Event.CASCADE_CHOSEN);
			}

		public MixCascade getChosenCascade()
			{
				return mConfig.getChosenMixCascade();
			}

		@Override
		/** Fetches the current cascades from the IS.
		 * @param bWithStatusUpdate if the current  cascades should be fetched as well.
		 * @param bOnlyVPN only returns cascades which support VPN.
		 */ public Hashtable getCascadeList(boolean bWithStatusUpdate, boolean bOnlyVPN)
			{
				return mInfoService.fetchMixCascades(bWithStatusUpdate, bOnlyVPN);
			}

		public void stopService()
			{
				mVpnService.shutdown();
				stopFGService();
				mDispatcher.notify(Event.SERVICE_SHUTDOWN);
			}

		/**
		 * Sends an intent to the Main Activity.
		 *
		 * @param code 0: Cascade connected; 1: Cascade disconnected.
		 */
		private void sendIntentToMainActivity(int code)
			{
//			Intent i = new Intent(getActivity().getApplicationContext(), MainActivity.class);
//			i.setAction(Intent.ACTION_MAIN);
//			i.setAction(Intent.CATEGORY_LAUNCHER);
//			i.putExtra("FragmentID", 0);
//			getActivity().startActivity(i);
			}

		@Override
		public void enableTunneling()
			{
				mVpnService.startTunnelingDevice();
				mDispatcher.notify(Event.VPN_ENABLED);
			}

		@Override
		public void disableTunneling()
			{
				mVpnService.stopTunnelingDevice();
				mDispatcher.notify(Event.VPN_DISABLED);
			}

		@Override
		public boolean tunnelingEnabled()
			{
				return mVpnService.tunnelingEnabled();
			}

		@Override
		public void notify(Event event)
			{
				// TODO: SEND INTENTS TO MAINACTIVITY HERE
				if (event == Event.CASCADE_CONNECTED)
					{
						Log.i(LOGTAG, "Event received: Cascade connected.");
						mConnected = true;
						Intent startIntent = new Intent(mVpnService.getApplicationContext(), MainActivity.class);
						startIntent.putExtra("FragmentID", MainActivity.FRAGMENTID_ANONYMITY);
						startIntent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
						mVpnService.startActivity(startIntent);

					}
				else if (event == Event.CASCADE_DISCONNECTED)
					{
						Log.i(LOGTAG, "Event received: Cascade disconnected.");
						mConnected = false;

						Intent startIntent = new Intent(mVpnService.getApplicationContext(), MainActivity.class);
						startIntent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
						mVpnService.startActivity(startIntent);

					}
				else if (event == Event.VPN_ENABLED)
					{
						Log.i(LOGTAG, "Event received: VPN enabled.");

					}
				else if (event == Event.VPN_DISABLED)
					{
						Log.i(LOGTAG, "Event received: VPN disabled.");

					}
				else
					{
						Log.e(LOGTAG, "Unexpected event!");
						throw new RuntimeException();
					}
				updateFGService();
				//todo: intent --> MainActivity->AnonFragment->updateStatus()
			}

		private void updateFGService()
			{
				Log.e("AndroidVPNMainActivity", "Preparing ForegroundService!");
//			mSharedPreferences.edit().putBoolean("FGServiceStarted", true).apply();
				Intent startIntent = new Intent(mVpnService.getApplicationContext(), ForegroundService.class);
				try
					{
						int conn_state;
						if (!mVpnService.tunnelingEnabled())
							{
								conn_state = 0;
							}
						else if (!mConnected)
							{
								conn_state = 1;
							}
						else
							{
								conn_state = 2;
								MixCascade cascade = getChosenCascade();
								String ip = "";
								String location = "";
								String name = "";
								if (cascade != null)
									{
										try
											{
												int nrOfMixes = cascade.getNumberOfMixes();
												ip = cascade.getMixInfo(nrOfMixes - 1).getFirstHostName();
												String countryCode=cascade.getMixInfo(nrOfMixes - 1).getServiceLocation().getCountryCode();
												CountryMapper c=new CountryMapper(countryCode);
												location = c.toString();
												name = getChosenCascade().getName();
											}
										catch (Exception e)
											{
												e.printStackTrace();
											}
									}
								startIntent.putExtra("ip", ip);
								startIntent.putExtra("location", location);
								startIntent.putExtra("name", name);

								startIntent.putExtra("conn_state", conn_state);
							}
					}
				catch (Exception e)
					{
						e.printStackTrace();
					}
				mVpnService.startService(startIntent);
			}

		private void stopFGService()
			{
				Intent stopIntent = new Intent(mVpnService.getApplicationContext(), ForegroundService.class);
				mVpnService.stopService(stopIntent);
			}
	}
