package anonvpn.anon_next.android.service.persistence;

import android.content.SharedPreferences;

import org.w3c.dom.Document;

import java.io.StringReader;

import anon.infoservice.MixCascade;
import anon.util.XMLUtil;
import anonvpn.anon_next.core.persistence.IConfig;


public class AndroidConfig implements IConfig {

    private SharedPreferences mSettings = null;

    private int mSentBytes = 0;
    private int mReceivedBytes = 0;

    // Internal variable used to avoid redundant parsing of
    // XML in case getChosenCascade gets called multiple times
    private MixCascade mChosenCascade = null;

    public AndroidConfig(SharedPreferences prefs) {
        mSettings = prefs;
    }

    @Override
    public void setString(String key, String data) {
        mSettings.edit().putString(key, data).commit();
    }

    @Override
    public void setInt(String key, int data) {
        mSettings.edit().putInt(key, data).commit();
    }

    @Override
    public String getString(String key, String defaultValue) {
        return mSettings.getString(key, defaultValue);
    }

    @Override
    public int getInt(String key, int defaultValue) {
        return mSettings.getInt(key, defaultValue);
    }

    @Override
    public MixCascade getChosenMixCascade() {

        MixCascade result = null;

        if (mChosenCascade == null) {
            String xml = this.getString("ChosenCascade.Object", "");
            if (!xml.equals("")) { // something was cached in SharedPrefs
                try {
                    Document doc = anon.util.XMLUtil.readXMLDocument(new StringReader(xml));
                    result = new MixCascade(doc.getDocumentElement());
                    mChosenCascade = result;
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        } else {
            result = mChosenCascade;
        }

        return result;
    }

    @Override
    public void setChosenMixCascade(MixCascade cascade) {

        String xml = XMLUtil.toString(cascade);
        this.setString("ChosenCascade.Object", xml);
        mChosenCascade = cascade;
    }

    @Override
    public String getChosenCascadeCredential() {
        return this.getString("ChosenCascade.Credential", "");
    }

    @Override
    public void setChosenCascadeCredential(String credential) {
        this.setString("ChosenCascade.Credential", credential);
    }

    @Override
    public synchronized void addSentBytes(int bytes) {
        mSentBytes += bytes;
    }

    /**
     * IMPORTANT: The number of bytes sent might overflow after some time.
     * This case must be handled by the callee of this method.
     *
     * @return int
     */
    @Override
    public int getSentBytes() {
        return mSentBytes;
    }

    @Override
    public synchronized void addReceivedBytes(int bytes) {
        mReceivedBytes += bytes;
    }

    /**
     * IMPORTANT: The number of bytes received might overflow after some time.
     * This case must be handled by the callee of this method.
     *
     * @return int
     */
    @Override
    public int getReceivedBytes() {
        return mReceivedBytes;
    }
}
