package anonvpn.anon_next.android.ui.main;


import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.os.Handler;
import android.support.v4.app.Fragment;
import android.support.v7.app.AlertDialog;
import android.support.v7.widget.SwitchCompat;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.CompoundButton;
import android.widget.ImageView;
import android.widget.ProgressBar;
import android.widget.TextView;

import anonvpn.anon_next.android.R;
import anonvpn.anon_next.android.service.gui.AndroidGUIInterface;
import anonvpn.anon_next.android.ui.HelpActivity;

/**
 * A simple {@link Fragment} subclass.
 */
public class AnonFragment extends Fragment
	{

		// TODO: Consider creating a global singleton for access to preferences instead of having a separate
		//       reference per activity/fragment etc.
		private SharedPreferences mPreferences;

		ProgressBar stat;
		TextView status;
		TextView location;
		View view;
		SwitchCompat sw;

		private final String LOGTAG = "AnonFragment.java";

		private final Handler mTrafficStatisticsHandler = new Handler();
		private final GetTrafficStatisticsTask mTrafficStatisticsTask = new GetTrafficStatisticsTask();
		private final int TRAFFIC_STATS_DELAY = 3000;

		private class AnonButtonChangeListener implements CompoundButton.OnCheckedChangeListener
			{

				private void showConfirmationDialog()
					{

						final AlertDialog dialog = new AlertDialog.Builder(getContext()).create();
						dialog.setTitle(getString(R.string.dialog_title_deact));
						dialog.setMessage(getString(R.string.dialog_message_deact));

						dialog.setButton(AlertDialog.BUTTON_POSITIVE, getString(R.string.dialog_b_yes), new DialogInterface.OnClickListener()
							{
								@Override
								public void onClick(DialogInterface dialogInterface, int i)
									{
										stopVPNTunneling();
										dialog.dismiss();
									}
							});

						dialog.setButton(AlertDialog.BUTTON_NEGATIVE, getString(R.string.dialog_b_no), new DialogInterface.OnClickListener()
							{
								@Override
								public void onClick(DialogInterface dialogInterface, int i)
									{
										sw.setChecked(true);
										dialog.dismiss();
									}
							});

						dialog.setCanceledOnTouchOutside(false);
						dialog.show();
					}

				@Override
				public void onCheckedChanged(CompoundButton buttonView, boolean isChecked)
					{
						if (isChecked)
							{
								// If the user canceled the confirmation dialog, the slide button changes
								// to checked although the VPN connection is still active
								if (((MainActivity) getActivity()).isTunnelingEnabled() == false)
									{

										if (!((MainActivity) getActivity()).getServiceInterface().hasChosenCascade()) {
											System.out.println("NO CASCADE HAS BEEN CHOSEN YET");
											Intent i = new Intent(getActivity().getApplicationContext(), MainActivity.class);
											i.setAction(Intent.ACTION_MAIN);
											i.setAction(Intent.CATEGORY_LAUNCHER);
											i.putExtra("FragmentID", 1);
											getActivity().startActivity(i);

											buttonView.setChecked(false);

										} else {

											startVPNTunneling();
											updateStatus();
										}

									}
							}
						else
							{
								// Check if the user canceled the passphrase form
								if (((MainActivity) getActivity()).isVPNConnected())
									{
										showConfirmationDialog();
									}
							}
					}
			}

		;

		private class GetTrafficStatisticsTask implements Runnable
			{

				private int tx = 0;
				private int rx = 0;
				private AndroidGUIInterface guiInterface = null;
				public boolean isScheduled = false;

				public void run()
					{
						guiInterface = ((MainActivity) getActivity()).getServiceInterface();

						// This tasks starts right after the VPN service gets started.
						// Therefore, the service might still be starting (hence, the interface is not ready)
						// when this run-procedure gets called.
						if (guiInterface != null)
							{
								tx = guiInterface.getTx();
								rx = guiInterface.getRx();

								updateTrafficStatistics(tx, rx);
							}

						mTrafficStatisticsHandler.postDelayed(this, TRAFFIC_STATS_DELAY);
					}

			}

		private void startVPNTunneling()
			{

				((MainActivity) getActivity()).enableVPNTunneling();

			}

		private void stopVPNTunneling()
			{

				((MainActivity) getActivity()).disableVPNTunneling();
				updateStatus();
			}

		private void updateTrafficStatistics(int tx, int rx)
			{

				final TextView up = (TextView) view.findViewById(R.id.anon_tv_upload);
				final TextView down = (TextView) view.findViewById(R.id.anon_tv_download);

				if (tx < 1000) {
					up.setText(tx + " B/s");
				} else if (tx < 1000000) {
					up.setText(tx / 1000 + " kB/s");
				} else {
					up.setText(tx / 1000000 + " MB/s");
				}

				if (rx < 1000) {
					down.setText(rx + " B/s");
				} else if (rx < 1000000) {
					down.setText(rx / 1000 + " kB/s");
				} else {
					down.setText(rx / 1000000 + " MB/s");
				}

			}


		@Override
		public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState)
			{

				mPreferences = getContext().getSharedPreferences(getContext().getString(R.string.sp_global), Context.MODE_PRIVATE);

				// Inflate the layout for this fragment
				view = inflater.inflate(R.layout.fragment_anon, container, false);

				status = (TextView) view.findViewById(R.id.anon_tv_status);
				ImageView anonHelp = (ImageView) view.findViewById(R.id.anon_iv_anonymometer_help);

				//assign Location
				location = (TextView) view.findViewById(R.id.anon_tv_location);

				anonHelp.setOnClickListener(new View.OnClickListener()
					{
						@Override
						public void onClick(View view)
							{
								Intent helpIntent = new Intent(getContext(), HelpActivity.class);
								getContext().setTheme(R.style.AppTheme);
								getContext().startActivity(helpIntent);
							}
					});

				//Anon-Button
				sw = (SwitchCompat) view.findViewById(R.id.anon_switch);
				//Do something on checked changed of ANON Switch
				sw.setOnCheckedChangeListener(new AnonButtonChangeListener());

				Log.e("ANONFragment", "ONCREATEVIEW");

				updateStatus();

				return view;
			}

		@Override
		public void onResume()
			{
				super.onResume();
				updateStatus();
			}

		@Override
		public void onPause()
			{
				super.onPause();
				mTrafficStatisticsHandler.removeCallbacks(mTrafficStatisticsTask);
				mTrafficStatisticsTask.isScheduled = false;
			}

		public void updateStatus()
			{

				if (((MainActivity) getActivity()).isTunnelingEnabled())
					{

						Log.e("ANONFragment", "VPN is running!");
						sw.setChecked(true);

						// Check if the user already selected a cascade
						if (((MainActivity) getActivity()).getServiceInterface().isConnectedToCascade())
							{

								if (mTrafficStatisticsTask.isScheduled == false)
									{
										mTrafficStatisticsHandler.postDelayed(mTrafficStatisticsTask, TRAFFIC_STATS_DELAY);
										mTrafficStatisticsTask.isScheduled = true;
									}

								status.setText(getContext().getString(R.string.start_state_active));
								status.setTextColor(getResources().getColor(R.color.connectivity_ok));
								((TextView) view.findViewById(R.id.anon_tv_tip)).setText("");
								location.setText(((MainActivity) getActivity()).getServiceInterface().getChosenCascade().getMixInfo(0).getName());

							}
						else {

							status.setText(getContext().getString(R.string.start_label_noconnect));
							status.setTextColor(getResources().getColor(R.color.connectivity_problem));
							((TextView) view.findViewById(R.id.anon_tv_tip)).setText(R.string.start_tip_choosenet);
							((ProgressBar) view.findViewById(R.id.anon_iv_statusbar)).setProgress(0);
						}

					}
				else
					{
						Log.e("ANONFragment", "VPN is not running!");
						sw.setChecked(false);

						mTrafficStatisticsHandler.removeCallbacks(mTrafficStatisticsTask);
						mTrafficStatisticsTask.isScheduled = false;
						updateTrafficStatistics(0, 0);

						status.setText(getContext().getString(R.string.start_state_deactive));
						status.setTextColor(getResources().getColor(R.color.connectivity_off));
						((TextView) view.findViewById(R.id.anon_tv_tip)).setText("");
						((ProgressBar) view.findViewById(R.id.anon_iv_statusbar)).setProgress(0);
						location.setText(getContext().getString(R.string.start_loc_unknown));
					}
			}
	};
