package anonvpn.anon_next.android.ui.main;

import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Bitmap;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.Drawable;
import android.os.Bundle;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.SwitchCompat;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.CompoundButton;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import anonvpn.anon_next.android.R;
import anonvpn.anon_next.android.ui.main.AppDataHolder;
import anonvpn.anon_next.android.ui.AppDetailActivity;

import java.util.ArrayList;

//Der Adapter verbindet View und DataModel miteinander
public class AppRecyclerViewAdapter extends RecyclerView.Adapter<AppRecyclerViewAdapter.ViewHolder> {

    private final ArrayList<AppDataHolder> _dataList;
    private Context _context;

    public AppRecyclerViewAdapter(ArrayList<AppDataHolder> items, Context context) {
        _dataList = items;
        _context = context;
    }

    @Override
    public ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext())
                .inflate(R.layout.item_app, parent, false);
        return new ViewHolder(view);
    }

    @Override
    public void onBindViewHolder(final ViewHolder holder, final int position) {
        //Speicherbereich wird zugewiesen durch String und privat (=> nur ANON kommt an die Daten darin)
        final SharedPreferences preferences = _context.getSharedPreferences(_context.getString(R.string.sp_global), Context.MODE_PRIVATE);
        holder._item = _dataList.get(position);
        holder._tv_appname.setText(_dataList.get(position).getName());
        holder._iv_appicon.setImageDrawable(holder._item.getIcon());
        holder._itemswitch.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                preferences.edit().putBoolean(_context.getString(R.string.sp_appchecked)+holder._item.getName(), isChecked).apply();
                holder._item.setChecked(isChecked);
            }
        });
        holder._itemswitch.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                boolean check = preferences.getBoolean(_context.getString(R.string.sp_appchecked)+holder._item.getName(), true);
                if(check) {
                    Toast.makeText(_context, holder._item.getName() + " " + _context.getString(R.string.app_toast_confirm), Toast.LENGTH_SHORT).show();
                }
                else{
                    Toast.makeText(_context, holder._item.getName() + " " + _context.getString(R.string.app_toast_dismiss), Toast.LENGTH_SHORT).show();
                }
            }
        });
        holder._view.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                //_context ist Activity main und es soll von dort aus AppDetail geöffnet werden
                Intent detailIntent = new Intent(_context, AppDetailActivity.class);
                Bundle arguments = new Bundle();
                arguments.putStringArray("permissions", holder._item.getPermList());
                arguments.putString("title", holder._item.getName());
                arguments.putLong("rxBytes", holder._item.getReceived());
                arguments.putLong("txBytes", holder._item.getTransmitted());
                //putDrawable
                Drawable icon = holder._item.getIcon();
                Bitmap bitmap = ((BitmapDrawable)icon).getBitmap();
                detailIntent.putExtra("Bitmap", bitmap);
                detailIntent.putExtras(arguments);
                _context.startActivity(detailIntent);
            }
        });
        boolean isChecked = preferences.getBoolean(_context.getString(R.string.sp_appchecked)+holder._item.getName(), true);
        holder._itemswitch.setChecked(isChecked);
    }

    @Override
    public int getItemCount() {
        return _dataList.size();
    }

    class ViewHolder extends RecyclerView.ViewHolder {
        final View _view;
        final TextView _tv_appname;
        final ImageView _iv_appicon;
        final SwitchCompat _itemswitch;
        AppDataHolder _item;

        ViewHolder(View view) {
            super(view);
            _view = view;
            _tv_appname = (TextView) view.findViewById(R.id.app_tv_name);
            _iv_appicon = (ImageView) view.findViewById(R.id.app_iv_icon);
            _itemswitch = (SwitchCompat) view.findViewById(R.id.app_switch);
        }
    }
}
