package anonvpn.anon_next.android.ui.main;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.net.VpnService;
import android.os.Bundle;
import android.os.Handler;
import android.support.design.widget.NavigationView;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.support.v4.view.GravityCompat;
import android.support.v4.widget.DrawerLayout;
import android.support.v7.app.ActionBarDrawerToggle;
import android.support.v7.widget.Toolbar;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;

import org.osmdroid.config.Configuration;

import anon.infoservice.MixCascade;
import anon.util.XMLUtil;
import anonvpn.anon_next.android.R;
import anonvpn.anon_next.android.ui.AboutActivity;
import anonvpn.anon_next.android.ui.HelpActivity;
import anonvpn.anon_next.android.ui.SettingsActivity;
import anonvpn.anon_next.android.ui.VPNConnectedActivity;
import anonvpn.anon_next.android.ui.tutorial.TutorialActivity;
import anonvpn.anon_next.core.CascadeConnectionManager;

//Main Activity of ANONmobile
public class MainActivity extends VPNConnectedActivity implements
		NavigationView.OnNavigationItemSelectedListener
	{

		private final String LOGTAG = this.getClass().getSimpleName();
		public static final int FRAGMENTID_ANONYMITY = 0;
		public static final int FRAGMENTID_CONNECTION = 1;
		public static final int FRAGMENTID_APPS = 2;
		public static final int FRAGMENTID_NETWORKS = 3;
		private int mActiveFragment = -1;
		private final int ALLOW_VPN_REQUEST = 0;
		private final int CHOOSE_CONNECTION_REQUEST = 1;

		private FragmentManager mFragmentManager = null;
		private SharedPreferences mSharedPreferences = null;

		@Override
		protected void onStart() {
			super.onStart();

			startVPNService();
		}

		@Override
		protected void onPause() {
			super.onPause();
			Log.d("ANONVPNMainActivity", "ONPAUSE");
		}

		@Override
		protected void onStop() {
			super.onStop();

			// If we are finishing, we will stop
			// the service during onDestroy
			if (! isFinishing()) {
				disconnectVPNService();
			}
			Log.d("ANONVPNMainActivity", "ONSTOP");
		}

		@Override
		protected void onCreate(Bundle savedInstanceState) {

            Log.d("ANONVPNMainActivity", "ONCREATE");
            super.onCreate(savedInstanceState);
            mFragmentManager = getSupportFragmentManager();

            Configuration.getInstance().setOsmdroidTileCache(getFilesDir());

            ///TODO: Move on better place...
            CascadeConnectionManager.USE_FAKE_DNS=false;
            CascadeConnectionManager.USE_UDP_SOCKS=true;

            mSharedPreferences = getSharedPreferences(this.getString(R.string.sp_global), Context.MODE_PRIVATE);

			if (mSharedPreferences.getBoolean("TutorialPassed", false) == false)
			{
				Intent intent = new Intent(this, TutorialActivity.class);
				intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP | Intent.FLAG_ACTIVITY_NEW_TASK);
				startActivity(intent);
			}
		}

		public void onNewIntent(Intent intent)
			{
				String LOGTAG = "MainActivity";
				Log.i(LOGTAG, "Received new intent");
				int target_fragment = intent.getIntExtra("FragmentID", -1);
				if (target_fragment != -1) {
					// On redirect from connection overview to anon fragment also check WAITING_FOR_CASCADE_CONNECTED
					// TODO: Else display an error message after a timeout
					if (mActiveFragment == FRAGMENTID_CONNECTION && target_fragment == FRAGMENTID_ANONYMITY) {
						ConnectFragment current_fragment = (ConnectFragment) mFragmentManager.findFragmentById(R.id.fl_fragmentcontainer);
						if (current_fragment != null && current_fragment.WAITING_FOR_CASCADE_CONNECTED) {
							current_fragment.WAITING_FOR_CASCADE_CONNECTED = false;
							setFragment(target_fragment);
						}
					} else {
						setFragment(target_fragment);
					}
				}
				else{
//					launchMainView();
				}
			}


		@Override
		public void onDestroy()
			{
				super.onDestroy();
				Log.d("ANONVPNMainActivity", "ONDESTROY");

				if (isFinishing() && isVPNConnected()) {
					stopVPNService();
				}
			}

		private void launchMainView()
			{

				setContentView(R.layout.activity_main);
				Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
				setSupportActionBar(toolbar);

				DrawerLayout drawer = (DrawerLayout) findViewById(R.id.drawer_layout);
				ActionBarDrawerToggle toggle = new ActionBarDrawerToggle(this, drawer, toolbar, R.string.navigation_drawer_open, R.string.navigation_drawer_close);
				drawer.setDrawerListener(toggle);
				toggle.syncState();

				NavigationView navigationView = (NavigationView) findViewById(R.id.nav_view);
				navigationView.setNavigationItemSelectedListener(this);

				setFragment(FRAGMENTID_ANONYMITY);

				if (mSharedPreferences.getBoolean("VPNStarted", false) && ! getServiceInterface().tunnelingEnabled())	{
					enableVPNTunneling();
				} else {
					Fragment fragment = mFragmentManager.findFragmentById(R.id.fl_fragmentcontainer);
					if (fragment != null) {

						if (fragment instanceof AnonFragment) {
							((AnonFragment) fragment).updateStatus();
						}
					}
				}
			}

		public void enableVPNTunneling() {

			{

				Log.e("AndroidVPNMainActivity", "Preparing VPN service!");
				mSharedPreferences.edit().putBoolean("VPNStarted", true).commit();
				// - Call prepare to ask for user permission
				Intent intent = VpnService.prepare(getApplicationContext());

				// Check if the VPN service has already been allowed earlier
				if (intent == null)
				{
					onVPNGranted();
				}
				else
				{
					startActivityForResult(intent, ALLOW_VPN_REQUEST);
				}

			}
		}

		private void onVPNGranted() {
			Log.e("AndroidVPNMainActivity", "Enable tunneling device!");
			getServiceInterface().enableTunneling();

			Fragment fragment = mFragmentManager.findFragmentById(R.id.fl_fragmentcontainer);
			if (fragment instanceof AnonFragment && fragment != null) {
				((AnonFragment) fragment).updateStatus();
			}
		}

		public void disableVPNTunneling() {

			Log.e("AndroidVPNMainActivity", "Disable tunneling device!");
			getServiceInterface().disableTunneling();
			mSharedPreferences.edit().putBoolean("VPNStarted", false).commit();

		}

		@Override
		protected void stopVPNService() {
			super.stopVPNService();
			mSharedPreferences.edit().putBoolean("VPNStarted", false).commit();
		}


		public void cascadeChosen() {
			setFragment(FRAGMENTID_ANONYMITY);
		}

		@Override
		protected void onVPNServiceConnected() {

			launchMainView();

		}

		protected void onVPNServiceDisconnected() {
			if (mActiveFragment == FRAGMENTID_ANONYMITY)
			{
				AnonFragment anonFragment = (AnonFragment) mFragmentManager.findFragmentById(R.id.fl_fragmentcontainer);

				// If the application has been terminated, the anonFragment might have already been destroyed
				if (anonFragment != null) {
					anonFragment.updateStatus();
				}
			}
		}

		@Override
		public void onActivityResult(int request_code, int result, Intent data)
			{
				if (request_code == ALLOW_VPN_REQUEST) {
					if (result == Activity.RESULT_OK)
					{
						onVPNGranted();

					}
					else if (result == Activity.RESULT_CANCELED)
					{

						// TODO: Launch an appropriate activity or fragment where the user can re-enable the VPN

					}
					else
					{

						// TODO: Check if this case can occur and determine appropriate behavior

					}
				} else if (request_code == CHOOSE_CONNECTION_REQUEST && result == RESULT_OK) {
					Log.w(LOGTAG, "Unexpected request_code: " + request_code);
				}
			}


		@Override
		protected void onResume()
			{
				super.onResume();
				Log.d("ANONVPNMainActivity", "ONRESUME");
				// TODO: Check if the following code is really needed
				/**
				 if (_preferences.getBoolean("tutorial_set", true))
				 {
				 Intent intent = new Intent(this, TutorialActivity.class);
				 intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP | Intent.FLAG_ACTIVITY_NEW_TASK);
				 startActivity(intent);
				 }
				 */
			}

		@Override
		public boolean onCreateOptionsMenu(Menu menu)
			{
				// Inflate the menu; this adds items to the action bar if it is present.
				getMenuInflater().inflate(R.menu.main, menu);
				return true;
			}

		@Override
		public boolean onOptionsItemSelected(MenuItem item)
			{
				// Handle action bar item clicks here. The action bar will
				// automatically handle clicks on the Home/Up button, so long
				// as you specify a parent activity in AndroidManifest.xml.
				int id = item.getItemId();

				if (id == R.id.options)
					{
						Intent intent = new Intent(this, SettingsActivity.class);
						intent.putExtra(SettingsActivity.EXTRA_SHOW_FRAGMENT, SettingsActivity.GeneralPreferenceFragment.class.getName());
						intent.putExtra(SettingsActivity.EXTRA_NO_HEADERS, true);
						startActivity(intent);
					}

				else if (id == R.id.action_help)
					{
						Intent intent = new Intent(this, HelpActivity.class);
						startActivity(intent);
					}

				else if (id == R.id.action_info)
					{
						Intent intent = new Intent(this, AboutActivity.class);
						startActivity(intent);
					}

//				else if (id == R.id.action_exit)
//					{
//						// Kill VPN Connection and exit app (Show dialog before?)
//						stopVPNService();
//						finish();
//					}

				return super.onOptionsItemSelected(item);
			}

		@Override
		public boolean onNavigationItemSelected(MenuItem item)
			{
				// Handle navigation view item clicks here.
				switch (item.getItemId())
					{
						case (R.id.anonymity):
							setFragment(FRAGMENTID_ANONYMITY);
							break;
						case (R.id.connection):
							setFragment(FRAGMENTID_CONNECTION);
							break;
					}

				DrawerLayout drawer = (DrawerLayout) findViewById(R.id.drawer_layout);
				drawer.closeDrawer(GravityCompat.START);
				return true;
			}

		/**
		 * This method is used by the ConnectFragment to resume to the anonymity fragment
		 * after a new cascade has been selected.
		 */
		public void showOverview() {
			setFragment(FRAGMENTID_ANONYMITY);
		}

		private void setFragment(final int id)
			{
				new Handler().post(new Runnable()
					{
						public void run()
							{
								mActiveFragment = id;
								switch (id)
									{
										case FRAGMENTID_ANONYMITY:
											mFragmentManager.beginTransaction().replace(R.id.fl_fragmentcontainer, new AnonFragment()).commit();
											setTheme(R.style.AppTheme_SwitchBig);
											setTitle(R.string.start_title);
											break;
										case FRAGMENTID_CONNECTION:
											mFragmentManager.beginTransaction().replace(R.id.fl_fragmentcontainer, new ConnectFragment()).commit();
											setTheme(R.style.AppTheme);
											setTitle(R.string.connect_title);
											break;
										case FRAGMENTID_APPS:
											mFragmentManager.beginTransaction().replace(R.id.fl_fragmentcontainer, new AppFragment()).commit();
											setTheme(R.style.AppTheme);
											setTitle(R.string.app_title);
											break;
										case FRAGMENTID_NETWORKS:
											mFragmentManager.beginTransaction().replace(R.id.fl_fragmentcontainer, new NetFragment()).commit();
											setTheme(R.style.AppTheme);
											setTitle(R.string.net_title);
											break;
									}
							}
					});
			}

		@Override
		public void onBackPressed()
		{
			DrawerLayout drawer = (DrawerLayout) findViewById(R.id.drawer_layout);
			if (drawer.isDrawerOpen(GravityCompat.START))
			{
				drawer.closeDrawer(GravityCompat.START);
				return;
			}

			Fragment activeFragment = mFragmentManager.findFragmentById(R.id.fl_fragmentcontainer);
			if (activeFragment != null && activeFragment instanceof ConnectFragment) {
				((ConnectFragment) activeFragment).onBackPressed();
			}
		}
	}
