package anonvpn.anon_next.android.ui.main;


import android.content.Context;
import android.graphics.drawable.Drawable;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.net.wifi.WifiConfiguration;
import android.net.wifi.WifiInfo;
import android.net.wifi.WifiManager;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.support.v7.widget.DividerItemDecoration;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.Toast;

import anonvpn.anon_next.android.R;
import anonvpn.anon_next.android.ui.main.NetDataHolder;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

/**
 * A simple {@link Fragment} subclass.
 */
public class NetFragment extends Fragment
	{

		private static ArrayList<NetDataHolder> _dataList = new ArrayList<>();
		private Context _context;
		private NetRecyclerViewAdapter _adapter;


		public NetFragment()
			{
			}

		@Override
		public void onCreate(Bundle savedInstanceState)
			{
				super.onCreate(savedInstanceState);

			}


		@Override
		public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState)
			{
				// Inflate the layout for this fragment
				View view = inflater.inflate(R.layout.fragment_net, container, false);

				_dataList.clear();

				_dataList = init(_context);

				Collections.sort(_dataList, new Comparator<NetDataHolder>()
					{
						@Override
						public int compare(NetDataHolder n1, NetDataHolder n2)
							{
								return n1.getName().compareToIgnoreCase(n2.getName());
							}
					});

				_adapter = new NetRecyclerViewAdapter(_dataList, getActivity());

				// Set the adapter
				Context context = view.getContext();
				RecyclerView recyclerView = (RecyclerView) view.findViewById(R.id.connect_list);
				recyclerView.setLayoutManager(new LinearLayoutManager(context));
				DividerItemDecoration dividerItemDecoration = new DividerItemDecoration(recyclerView.getContext(), ((LinearLayoutManager) recyclerView.getLayoutManager()).getOrientation());
				recyclerView.addItemDecoration(dividerItemDecoration);
				recyclerView.setAdapter(_adapter);

				//Set Buttons
				Button checkButton = (Button) view.findViewById(R.id.net_b_anon);
				Button uncheckButton = (Button) view.findViewById(R.id.net_b_deanon);

				checkButton.setOnClickListener(new View.OnClickListener()
					{
						@Override
						public void onClick(View v)
							{
								checkAll();
							}
					});
				uncheckButton.setOnClickListener(new View.OnClickListener()
					{
						@Override
						public void onClick(View v)
							{
								uncheckAll();
							}
					});

				return view;

			}

		@Override
		public void onAttach(Context context)
			{
				super.onAttach(context);
				_context = context;
			}

		@Override
		public void onDetach()
			{
				super.onDetach();
			}

		private void checkAll()
			{
				Toast.makeText(_context, getString(R.string.net_button_anon), Toast.LENGTH_SHORT).show();
				for (NetDataHolder holder : _dataList)
					{
						holder.setChecked(true);
						_context.getSharedPreferences(_context.getString(R.string.sp_global), Context.MODE_PRIVATE).edit().putBoolean(_context.getString(R.string.sp_netchecked) + holder.getName(), true).apply();
						_adapter.notifyDataSetChanged();
					}
			}

		private void uncheckAll()
			{
				Toast.makeText(_context, getString(R.string.net_button_deanon), Toast.LENGTH_SHORT).show();
				for (NetDataHolder holder : _dataList)
					{
						holder.setChecked(false);
						_context.getSharedPreferences(_context.getString(R.string.sp_global), Context.MODE_PRIVATE).edit().putBoolean(_context.getString(R.string.sp_netchecked) + holder.getName(), false).apply();
						_adapter.notifyDataSetChanged();
					}
			}

		public static ArrayList<NetDataHolder> init(Context context)
			{

				ArrayList<NetDataHolder> dataList = new ArrayList<>();
				List<WifiConfiguration> pkgNetList = null;
				WifiManager manager = (WifiManager) context.getSystemService(Context.WIFI_SERVICE);
				if (manager != null)
					{
						pkgNetList = manager.getConfiguredNetworks();
					}
				if (pkgNetList != null)
					{
						for (WifiConfiguration currentInfo : pkgNetList)
							{

								String name = currentInfo.SSID.replace("\"", "");
								String security;
								Drawable icon;

								if (getSecurity(currentInfo) == 0)
									{
										security = context.getString(R.string.net_label_securepsk);
										icon = context.getResources().getDrawable(R.drawable.ic_signal_wifi_4_bar_lock_black_24dp);
									}
								else if (getSecurity(currentInfo) == 1)
									{
										security = context.getString(R.string.net_label_secureeap);
										icon = context.getResources().getDrawable(R.drawable.ic_signal_wifi_4_bar_lock_black_24dp);
									}
								else if (getSecurity(currentInfo) == 2)
									{
										security = context.getString(R.string.net_label_securewep);
										icon = context.getResources().getDrawable(R.drawable.ic_signal_wifi_4_bar_lock_black_24dp);
									}
								else
									{
										security = context.getString(R.string.net_label_unsave);
										icon = context.getResources().getDrawable(R.drawable.ic_perm_scan_wifi_black_24dp);
									}

								String currWifi = getCurrentSsid(context);
								if (currWifi != null)
									{
										if (name.equals(getCurrentSsid(context)))
											{
												dataList.add(0, new NetDataHolder(name, icon, security));

											}
										else
											{
												dataList.add(new NetDataHolder(name, icon, security));
											}
									}
								else
									{
										dataList.add(new NetDataHolder(name, icon, security));
									}
							}
					}
				//add mobile network dummy
				dataList.add(new NetDataHolder(context.getString(R.string.net_label_celname), context.getResources().getDrawable(R.drawable.ic_network_cell_black_24dp), context.getString(R.string.net_label_celsecure)));


				return dataList;
			}

		static int getSecurity(WifiConfiguration config)
			{
				if (config.allowedKeyManagement.get(WifiConfiguration.KeyMgmt.WPA_PSK))
					{
						return 0;
					}
				else if (config.allowedKeyManagement.get(WifiConfiguration.KeyMgmt.WPA_EAP) || config.allowedKeyManagement.get(WifiConfiguration.KeyMgmt.IEEE8021X))
					{
						return 1;
					}
				else if (config.wepKeys[0] != null)
					{
						return 2;
					}
				else
					{
						return 3;
					}
			}

		public static String getCurrentSsid(Context context)
			{
				String ssid = null;
				ConnectivityManager connManager = (ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE);
				NetworkInfo networkInfo = connManager.getNetworkInfo(ConnectivityManager.TYPE_WIFI);
				if (networkInfo.isConnected())
					{
						final WifiManager wifiManager = (WifiManager) context.getSystemService(Context.WIFI_SERVICE);
						final WifiInfo connectionInfo = wifiManager.getConnectionInfo();
						if (connectionInfo != null && !TextUtils.isEmpty(connectionInfo.getSSID()))
							{
								ssid = connectionInfo.getSSID().replace("\"", "");
							}
					}
				return ssid;
			}

	}
