package anonvpn.anon_next.android.ui.tutorial;


import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.pm.ResolveInfo;
import android.graphics.drawable.Drawable;
import android.net.TrafficStats;
import android.os.AsyncTask;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.support.v7.widget.DividerItemDecoration;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.Toast;

import anonvpn.anon_next.android.R;
import anonvpn.anon_next.android.ui.main.AppDataHolder;
import anonvpn.anon_next.android.ui.main.AppRecyclerViewAdapter;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

/**
 * A simple {@link Fragment} subclass.
 */
public class ChooseAppFragment extends Fragment
	{


		private static ArrayList<AppDataHolder> _dataList = new ArrayList<>();
		private Context _context;
		private AppRecyclerViewAdapter _adapter;

		/**
		 * Mandatory empty constructor for the fragment manager to instantiate the
		 * fragment (e.g. upon screen orientation changes).
		 */
		public ChooseAppFragment()
			{
			}

		@Override
		public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState)
			{
				View view = inflater.inflate(R.layout.fragment_app_list, container, false);

				new InitTask(_context).execute();

				//Collections.sort(mMixList, String.CASE_INSENSITIVE_ORDER);
				Collections.sort(_dataList, new Comparator<AppDataHolder>()
					{
						@Override
						public int compare(AppDataHolder n1, AppDataHolder n2)
							{
								return n1.getName().compareToIgnoreCase(n2.getName());
							}
					});

				_adapter = new AppRecyclerViewAdapter(_dataList, getActivity());

				// Set the adapter
				Context context = view.getContext();
				RecyclerView recyclerView = (RecyclerView) view.findViewById(R.id.connect_list);
				recyclerView.setLayoutManager(new LinearLayoutManager(context));
				DividerItemDecoration dividerItemDecoration = new DividerItemDecoration(recyclerView.getContext(), ((LinearLayoutManager) recyclerView.getLayoutManager()).getOrientation());
				recyclerView.addItemDecoration(dividerItemDecoration);
				recyclerView.setAdapter(_adapter);

				//Set Buttons
				Button checkButton = (Button) view.findViewById(R.id.app_b_anon);
				Button uncheckButton = (Button) view.findViewById(R.id.app_b_deanon);

				checkButton.setOnClickListener(new View.OnClickListener()
					{
						@Override
						public void onClick(View v)
							{
								checkAll();
							}
					});
				uncheckButton.setOnClickListener(new View.OnClickListener()
					{
						@Override
						public void onClick(View v)
							{
								uncheckAll();
							}
					});

				return view;
			}

		@Override
		public void onAttach(Context context)
			{
				super.onAttach(context);
				_context = context;
			}

		@Override
		public void onDetach()
			{
				super.onDetach();
			}

		private void checkAll()
			{
				Toast.makeText(_context, _context.getString(R.string.app_toast_active), Toast.LENGTH_SHORT).show();
				for (AppDataHolder holder : _dataList)
					{
						holder.setChecked(true);
						_context.getSharedPreferences(_context.getString(R.string.sp_global), Context.MODE_PRIVATE).edit().putBoolean(_context.getString(R.string.sp_appchecked) + holder.getName(), true).apply();
						_adapter.notifyDataSetChanged();
					}
			}

		private void uncheckAll()
			{
				Toast.makeText(_context, _context.getString(R.string.app_toast_deactive), Toast.LENGTH_SHORT).show();
				for (AppDataHolder holder : _dataList)
					{
						holder.setChecked(false);
						_context.getSharedPreferences(_context.getString(R.string.sp_global), Context.MODE_PRIVATE).edit().putBoolean(_context.getString(R.string.sp_appchecked) + holder.getName(), false).apply();
						_adapter.notifyDataSetChanged();
					}
			}

		private final class InitTask extends AsyncTask<Void, Integer, Void>
			{

				private ProgressDialog m_progressDialog;

				InitTask(Context context)
					{
						m_progressDialog = new ProgressDialog(context);
						m_progressDialog.setMessage(_context.getString(R.string.app_label_load));
					}

				@Override
				protected void onPreExecute()
					{
						super.onPreExecute();
						m_progressDialog.show();
					}

				@Override
				protected Void doInBackground(Void... voids)
					{
						_dataList.clear();

						Intent mainIntent = new Intent(Intent.ACTION_MAIN, null);
						mainIntent.addCategory(Intent.CATEGORY_LAUNCHER);

						List<ResolveInfo> pkgAppsList = _context.getPackageManager().queryIntentActivities(mainIntent, 0);
						int iter = 0;
						for (ResolveInfo currentInfo : pkgAppsList)
							{
								if (!currentInfo.loadLabel(_context.getPackageManager()).equals("ANONmobile"))
									{
										String name = currentInfo.loadLabel(_context.getPackageManager()) + "";
										Drawable icon = currentInfo.loadIcon(_context.getPackageManager());
										try
											{
												PackageInfo packageInfo = _context.getPackageManager().getPackageInfo(currentInfo.activityInfo.packageName, PackageManager.GET_PERMISSIONS);
												String[] permList = packageInfo.requestedPermissions;

												int uid = packageInfo.applicationInfo.uid;
												long txApp = (TrafficStats.getUidTxBytes(uid)) / 1000;
												long rxApp = (TrafficStats.getUidRxBytes(uid)) / 1000;
												if (txApp > 0 && rxApp > 0)
													{
														txApp = txApp / 1000;
														rxApp = rxApp / 1000;
													}

												//PermissionInfo[] permList = packageInfo.permissions;
												if (permList != null)
													{
														List<String> cleanedPerms = new ArrayList<>();
														for (String item : permList)
															{
																if (item != null)
																	{
																		String group = null;
																		try
																			{
																				group = _context.getPackageManager().getPermissionInfo(item, PackageManager.GET_META_DATA).group;
																			}
																		catch (PackageManager.NameNotFoundException e)
																			{
																				e.printStackTrace();
																			}
																		if (group != null && !cleanedPerms.contains(group))
																			{
																				cleanedPerms.add(group);
																			}
																	}
															}
														String[] cleanedArr = new String[cleanedPerms.size()];
														cleanedArr = cleanedPerms.toArray(cleanedArr);
														for (int i = 0; i < cleanedArr.length; i++)
															{
																String cleanedGroup = cleanedArr[i].toLowerCase();
																cleanedGroup = cleanedGroup.replace("_", " ");
																cleanedArr[i] = cleanedGroup;
															}
														publishProgress(iter, pkgAppsList.size());
														_dataList.add(new AppDataHolder(name, icon, txApp, rxApp, cleanedArr));
													}
											}
										catch (PackageManager.NameNotFoundException e)
											{
												e.printStackTrace();
											}
									}
								iter++;
							}
						//Collections.sort(mMixList, String.CASE_INSENSITIVE_ORDER);
						Collections.sort(_dataList, new Comparator<AppDataHolder>()
							{
								@Override
								public int compare(AppDataHolder n1, AppDataHolder n2)
									{
										return n1.getName().compareToIgnoreCase(n2.getName());
									}
							});
						return null;
					}

				@Override
				protected void onProgressUpdate(Integer... values)
					{
						super.onProgressUpdate(values);
						_adapter.notifyDataSetChanged();
						m_progressDialog.setMessage(_context.getString(R.string.app_label_load) + " " + (values[0] + 1) + "/" + values[1]);

					}

				@Override
				protected void onPostExecute(Void aVoid)
					{
						super.onPostExecute(aVoid);
						m_progressDialog.dismiss();
						m_progressDialog =null;
						_adapter.notifyDataSetChanged();
					}
			}

	}
