package anonvpn.anon_next.android.ui.tutorial;


import android.content.Context;
import android.graphics.drawable.Drawable;
import android.net.wifi.WifiConfiguration;
import android.net.wifi.WifiManager;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.support.v7.widget.DividerItemDecoration;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.Toast;

import anonvpn.anon_next.android.R;
import anonvpn.anon_next.android.ui.main.NetDataHolder;
import anonvpn.anon_next.android.ui.main.NetRecyclerViewAdapter;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

/**
 * A simple {@link Fragment} subclass.
 */
public class ChooseNetFragment extends Fragment {

    private static ArrayList<NetDataHolder> _dataList = new ArrayList<>();
    private Context _context;
    private NetRecyclerViewAdapter _adapter;


    public ChooseNetFragment() {
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        _dataList.clear();

        WifiManager manager = (WifiManager) _context.getSystemService(Context.WIFI_SERVICE);
        List<WifiConfiguration> pkgNetList = manager.getConfiguredNetworks();
        if(pkgNetList!=null) {
            for (WifiConfiguration currentInfo : pkgNetList) {

                String name = currentInfo.SSID.replace("\"", "");
                String security;
                Drawable icon;

                if (getSecurity(currentInfo) == 0) {
                    security = _context.getString(R.string.net_label_securepsk);
                    icon = getResources().getDrawable(R.drawable.ic_signal_wifi_4_bar_lock_black_24dp);
                } else if (getSecurity(currentInfo) == 1) {
                    security = _context.getString(R.string.net_label_secureeap);
                    icon = getResources().getDrawable(R.drawable.ic_signal_wifi_4_bar_lock_black_24dp);
                } else if (getSecurity(currentInfo) == 2) {
                    security = _context.getString(R.string.net_label_securewep);
                    icon = getResources().getDrawable(R.drawable.ic_signal_wifi_4_bar_lock_black_24dp);
                } else {
                    security = _context.getString(R.string.net_label_unsave);
                    icon = getResources().getDrawable(R.drawable.ic_perm_scan_wifi_black_24dp);
                }
                // currentInfo.loadIcon(_context.getPackageManager());
                _dataList.add(new NetDataHolder(name, icon, security));
            }
        }
        //add mobile network dummy
        _dataList.add(new NetDataHolder(_context.getString(R.string.net_label_celname), getResources().getDrawable(R.drawable.ic_network_cell_black_24dp), _context.getString(R.string.net_label_celsecure)));

    }


    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        View view = inflater.inflate(R.layout.fragment_net, container, false);

        Collections.sort(_dataList, new Comparator<NetDataHolder>(){
            @Override
            public int compare(NetDataHolder n1, NetDataHolder n2) {
                return n1.getName().compareToIgnoreCase(n2.getName());
            }
        });

        _adapter = new NetRecyclerViewAdapter(_dataList, getActivity());

        // Set the adapter
        Context context = view.getContext();
        RecyclerView recyclerView = (RecyclerView) view.findViewById(R.id.connect_list);
        recyclerView.setLayoutManager(new LinearLayoutManager(context));
        DividerItemDecoration dividerItemDecoration = new DividerItemDecoration(recyclerView.getContext(), ((LinearLayoutManager) recyclerView.getLayoutManager()).getOrientation());
        recyclerView.addItemDecoration(dividerItemDecoration);
        recyclerView.setAdapter(_adapter);

        //Set Buttons
        Button checkButton = (Button) view.findViewById(R.id.net_b_anon);
        Button uncheckButton = (Button) view.findViewById(R.id.net_b_deanon);

        checkButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                checkAll();
            }
        });
        uncheckButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                uncheckAll();
            }
        });

        return view;

    }

    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
        _context = context;
    }

    @Override
    public void onDetach() {
        super.onDetach();
    }

    private void checkAll(){
        Toast.makeText(_context, _context.getString(R.string.net_toast_active), Toast.LENGTH_SHORT).show();
        for (NetDataHolder holder:_dataList){
            holder.setChecked(true);
            _context.getSharedPreferences(_context.getString(R.string.sp_global), Context.MODE_PRIVATE)
                    .edit().putBoolean(_context.getString(R.string.sp_netchecked)+holder.getName(), true).apply();
            _adapter.notifyDataSetChanged();
        }
    }
    private void uncheckAll(){
        Toast.makeText(_context, _context.getString(R.string.net_toast_deactive), Toast.LENGTH_SHORT).show();
        for (NetDataHolder holder:_dataList){
            holder.setChecked(false);
            _context.getSharedPreferences(_context.getString(R.string.sp_global), Context.MODE_PRIVATE)
                    .edit().putBoolean(_context.getString(R.string.sp_netchecked)+holder.getName(), false).apply();
            _adapter.notifyDataSetChanged();
        }
    }

    static int getSecurity(WifiConfiguration config) {
        if (config.allowedKeyManagement.get(WifiConfiguration.KeyMgmt.WPA_PSK)) {
            return 0;
        }
        else if (config.allowedKeyManagement.get(WifiConfiguration.KeyMgmt.WPA_EAP) ||
                config.allowedKeyManagement.get(WifiConfiguration.KeyMgmt.IEEE8021X)) {
            return 1;
        }
        else if(config.wepKeys[0] != null){ return 2; }
        else{
            return 3;
        }
    }

}
