package anonvpn.anon_next.android.ui.tutorial;

import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentTransaction;
import android.support.v7.app.AlertDialog;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.ImageButton;
import android.widget.ProgressBar;

import org.osmdroid.util.GeoPoint;

import anon.infoservice.MixCascade;
import anon.infoservice.MixInfo;
import anon.util.XMLUtil;
import anonvpn.anon_next.android.R;
import anonvpn.anon_next.android.ui.VPNConnectedActivity;
import anonvpn.anon_next.android.ui.main.ConnectButtonFragment;
import anonvpn.anon_next.android.ui.main.ConnectDetailFragment;
import anonvpn.anon_next.android.ui.main.ConnectFragment;
import anonvpn.anon_next.android.ui.main.ConnectListFragment;
import anonvpn.anon_next.android.ui.main.MainActivity;

/**
 * The TutorialActivity is typically started on top of the MainActivity.
 * To avoid interference during pause/resume etc., the MainActivity handles
 * the connection to the VPN. The TutorialActivity never starts or stops
 * the VPN service, but assumes that the service is available during its full
 * lifetime.
 */
public class TutorialActivity extends VPNConnectedActivity {

    private FragmentManager mFragmentManager = getSupportFragmentManager();
    SharedPreferences _preferences;
    ProgressBar status;
    int currentFrag = 0;
    ImageButton backB;
    Button continueB;

    public static final int FRAGMENTID_WELCOME = 0;
    public static final int FRAGMENTID_INTROCONNECTION = 1;
    public static final int FRAGMENTID_CHOOSECONNECTION = 2;
    public static final int FRAGMENTID_FINISH = 5;

    private MixCascade mMarkedCascade;
    private MixCascade mDetailCascade;
    private ConnectFragment mConnectFragmentCached;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_tutorial);
        Log.d("ANONVPNTutorialActivity", "ONCREATE");
        _preferences = this.getSharedPreferences(this.getString(R.string.sp_global), MODE_PRIVATE);

        //assign first usage
        _preferences.edit().putBoolean("firstuse", true).apply();

        //Statusbar
        status = (ProgressBar) findViewById(R.id.tut_pb_statusbar);
        backB = (ImageButton) findViewById(R.id.tut_b_back);
        continueB = (Button) findViewById(R.id.tut_b_continue);

        setFragment(FRAGMENTID_WELCOME, true);

    }

    @Override
    protected void onStart() {
        super.onStart();
        Log.d("ANONVPNTutorialActivity", "ONSTART");
        startVPNService();
    }

    @Override
    protected void onStop() {
        super.onStop();
        Log.d("ANONVPNTutorialActivity", "ONSTOP");
        disconnectVPNService();
    }

    @Override
    protected void onVPNServiceConnected() {
        final Context c = this;
        findViewById(R.id.tut_b_continue).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {

                if (currentFrag == FRAGMENTID_CHOOSECONNECTION && _preferences.getInt(getString(R.string.sp_connectchecked), -1) == -1) {
                    AlertDialog dialog = new AlertDialog.Builder(c).create();
                    dialog.setTitle(c.getString(R.string.dialog_tut_nocont));
                    dialog.setMessage(c.getString(R.string.dialog_tut_noconm));
                    dialog.setButton(AlertDialog.BUTTON_POSITIVE, "Ok", new DialogInterface.OnClickListener() {
                        @Override
                        public void onClick(DialogInterface dialogInterface, int i) {
                        }
                    });
                    dialog.show();
                } else if (currentFrag != FRAGMENTID_FINISH) {
                    increaseProgress();
                    setFragment(currentFrag + 1, true);
                } else {
                    enableTunneling();
                    endTut();
                }
            }
        });

        findViewById(R.id.tut_b_back).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                decreaseProgress();
                setFragment(currentFrag - 1, false);
            }
        });
    }

    @Override
    protected void onVPNServiceDisconnected() {

    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        Log.d("ANONVPNTutorialActivity", "ONDESTROY");
    }

    public void cascadeChosen() {
        setFragment(FRAGMENTID_FINISH, true);
    }

    public void setFragment(int id, boolean isIncreased) {
        currentFrag = id;
        backB.setVisibility(View.VISIBLE);
        status.setVisibility(View.VISIBLE);
        continueB.setVisibility(View.VISIBLE);
        switch (id) {
            case FRAGMENTID_WELCOME: {
                mFragmentManager.beginTransaction().replace(R.id.fl_fragmentcontainer, new WelcomeFragment()).commit();
                backB.setVisibility(View.GONE);
                status.setVisibility(View.GONE);
                continueB.setVisibility(View.GONE);
                break;
            }
            case FRAGMENTID_INTROCONNECTION: {

                FragmentTransaction transaction = mFragmentManager.beginTransaction();

                transaction.replace(R.id.fl_fragmentcontainer, new IntroConnectionFragment()).commit();
                backB.setVisibility(View.GONE);
                break;
            }
            case FRAGMENTID_CHOOSECONNECTION: {
                FragmentTransaction transaction = mFragmentManager.beginTransaction();

                if (isIncreased) {
                    transaction.setCustomAnimations(R.anim.enter_from_right, R.anim.exit_to_left);
                } else {
                    transaction.setCustomAnimations(R.anim.enter_from_left, R.anim.exit_to_right);
                }
                transaction.replace(R.id.fl_fragmentcontainer, new ConnectFragment()).commit();
                break;
            }
            case FRAGMENTID_FINISH: {
                FragmentTransaction transaction = mFragmentManager.beginTransaction();

                if (isIncreased) {
                    transaction.setCustomAnimations(R.anim.enter_from_right, R.anim.exit_to_left);
                } else {
                    transaction.setCustomAnimations(R.anim.enter_from_left, R.anim.exit_to_right);
                }
                transaction.replace(R.id.fl_fragmentcontainer, new FinishFragment()).commit();
                break;
            }
        }
    }

    public void enableTunneling() {
        _preferences.edit().putBoolean("VPNStarted",true).commit();
    }

    public void endTut(){
        _preferences = this.getSharedPreferences(this.getString(R.string.sp_global), MODE_PRIVATE);
        _preferences.edit().putBoolean("TutorialPassed", true).apply();

        Intent intent = new Intent(this, MainActivity.class);
        intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP | Intent.FLAG_ACTIVITY_NEW_TASK);
        intent.putExtra("FragmentID",-1);
        startActivity(intent);
			}

    @Override
    public void onBackPressed() {

        Fragment activeFragment = mFragmentManager.findFragmentById(R.id.fl_fragmentcontainer);
        if (activeFragment != null && activeFragment instanceof ConnectDetailFragment && mConnectFragmentCached != null) {
            setTitle(R.string.connect_title);
            // Return the cached ConnectFragment
            mFragmentManager.beginTransaction().replace(R.id.fl_fragmentcontainer, mConnectFragmentCached).commit();
        } else if (currentFrag > 0) {
            decreaseProgress();
            setFragment(currentFrag - 1, false);
        }
    }
    
    private int progressSteps = 2;

    public void increaseProgress(){

        status.incrementProgressBy(((100 / progressSteps)));
    }

    public void decreaseProgress(){
        if(currentFrag == FRAGMENTID_CHOOSECONNECTION){
            status.setProgress(0);
        }
        else {
            status.incrementProgressBy(((-100 / progressSteps)));
        }
    }
	}
