package anonvpn.anon_next.android.util;

import android.content.res.AssetManager;

import java.io.File;
import java.io.InputStream;
import java.util.Hashtable;

import anon.util.IResourceInstantiator;
import anon.util.IResourceLoaderHelper;
import anon.util.ResourceLoader;

/**
 * Created by sk13 on 15.08.2017.
 */

public class AndroidResourceLoaderHelper implements IResourceLoaderHelper
	{

		private AssetManager m_AssetManager;

		public AndroidResourceLoaderHelper(AssetManager assetManager)
			{
				m_AssetManager = assetManager;
			}

		/**
		 * Loads resources from a given directory (simple file, directory or zip
		 * file) that are found in the given resource search path and instantiates
		 * them as objects of a type defined by the resource instantiator. Resources
		 * with the same path name are only loaded once according to the given
		 * Hashtable. The resource search path is relative to the given directory
		 * and may either specify a single resource or, if it ends with a slash "/",
		 * a (virtual) directory in the given directory. Therefore a resource search
		 * path "/" loads all resources in the specified directory. A resource
		 * search path "certificates/" loads all resources in the (virtual)
		 * directory <i> certificates </i>.
		 *
		 * @param a_strResourceSearchPath
		 *          a (virtual) path in the directory to load resources from
		 * @param a_Directory
		 *          a simple file, directory or zip file
		 * @param a_instantiator
		 *          a ResourceInstantiator that is used to instantiate the loaded
		 *          resources
		 * @param a_bRecursive
		 *          true if (virtual) subdirectories should be searched for
		 *          resources; false otherwise (has an effect only for resource
		 *          paths ending with "/")
		 * @param a_bStopAtFirstResource
		 *          true if the search should stop with the first loaded resource;
		 *          false otherwise
		 * @param a_loadedResources
		 *          a Hashtable where the loaded and instantiated resources are
		 *          stored
		 */
		public void loadResources(String a_strResourceSearchPath, File a_Directory, IResourceInstantiator a_instantiator,
															boolean a_bRecursive, boolean a_bStopAtFirstResource, Hashtable a_loadedResources)
			{
				if ((a_strResourceSearchPath = ResourceLoader.formatResourcePath(a_strResourceSearchPath)) == null
								|| a_loadedResources == null || a_instantiator == null)
					{
						return;
					}

				try
					{
						String[] entries;
						if (a_strResourceSearchPath.endsWith("/"))
							{
								entries = m_AssetManager.list(a_strResourceSearchPath.substring(0,a_strResourceSearchPath.length()-1));
							}
						else
							{
								entries = new String[1];
								entries[0] = a_strResourceSearchPath;
							}

						for (String entry : entries)
							{
								try
									{
										Object object = null;
										InputStream in = m_AssetManager.open(a_strResourceSearchPath+entry);
										try
											{
												object = a_instantiator.getInstance(in, entry);
												if (!a_loadedResources.containsKey(entry))
													{
														a_loadedResources.put(entry, object);
														if (!a_strResourceSearchPath.endsWith("/") || a_bStopAtFirstResource)
															{
																// the requested resource has been found
																return;
															}
													}
											}
										catch (Exception a_e)
											{
											}
										try
											{
												in.close();
											}
										catch (Exception e2)
											{
											}

									}
								catch (Exception a_e)
									{
									}
							}
					}
				catch (Exception a_e)
					{
						return;
					}
			}
	}
